/* RXECI3.CMD  */
/*******************************************************************/
/*                                                                 */
/* MODULE NAME      RXECI3.CMD                                     */
/*                                                                 */
/* DESCRIPTIVE NAME Demonstrates RxECI                             */
/*                                                                 */
/* Statement:       Licensed Materials - Property of IBM           */
/*                                                                 */
/*                  CA31 SupportPac                                */
/*                  (c) Copyright IBM Corp. 1994.                  */
/*                                                                 */
/*                  See Copyright Instructions.                    */
/*                                                                 */
/*                  All rights reserved.                           */
/*                                                                 */
/*                  U.S. Government Users Restricted Rights - use, */
/*                  duplication or disclosure restricted by GSA    */
/*                  ADP Schedule Contract with IBM Corp.           */
/*                                                                 */
/* Status:          Version 1 Release 0                            */
/*                                                                 */
/*  NOTES :-                                                       */
/*    DEPENDENCIES = OS/2 V2.x,CICS OS/2, RxECI                    */
/*                   None                                          */
/*    RESTRICTIONS = none                                          */
/*    MODULE TYPE  = Command file                                  */
/*    PROCESSOR    = PS/2 and PC                                   */
/*                                                                 */
/*******************************************************************/
/*                                                                 */
/* This program demonstrates a synchronous state call using RxECI. */
/* COMMAREA must be 10.                                            */
/*                                                                 */
/*******************************************************************/
  If rxfuncquery('ECI') Then Do
     If rxfuncadd('ECI', 'RXECI', 'RxECI' ) Then signal failure
     say "ECI registered"
  End

/* provide a simple COMMAREA just filled with '.' */
  commarea = copies('.',20)

/* length must be equal 10 */
  ECI.commarea_length   = "10"
  ECI.extend_mode       = "ECI_STATE_IMMEDIATE"
  ECI.call_type         = "ECI_STATE_SYNC"

  rc = show_eci_stem()
  commarea2 = ECI('ECI.',commarea)
  rc = check_rc(commarea2)
  parse value commarea2 with ConnectionType    3,
                             CicsServerStatus  5,
                             CicsClientStatus  7,
                             CicsTraceState    9,
                             CicsTraceFlags    11,
                             .;

  say "ConnectionType  " c2x(ConnectionType)
  say "CicsServerStatus" c2x(CicsServerStatus)
  say "CicsClientStatus" c2x(CicsClientStatus)
  say "CicsTraceState  " c2x(CicsTraceState)
  say "CicsTraceFlags  " c2x(CicsTraceFlags)

/*
   short      ConnectionType;
      #define     ECI_CONNECTED_NOWHERE         0
      #define     ECI_CONNECTED_TO_SERVER       1
      #define     ECI_CONNECTED_TO_CLIENT       2
   short      CicsServerStatus;
      #define     ECI_SERVERSTATE_UNKNOWN       0
      #define     ECI_SERVERSTATE_UP            1
      #define     ECI_SERVERSTATE_DOWN          2
   short      CicsClientStatus;
      #define     ECI_CLIENTSTATE_UNKNOWN       0
      #define     ECI_CLIENTSTATE_UP            1
      #define     ECI_CLIENTSTATE_INAPPLICABLE  2
   short      CicsTraceState;
      #define     ECI_TRACESTATE_UNKNOWN        0
      #define     ECI_TRACESTATE_ACTIVE         1
      #define     ECI_TRACESTATE_INACTIVE       2
   short      CicsTraceFlags;
      #define     ECI_SYSTEM_TRACE            0x1
      #define     ECI_USER_TRACE              0x2
      #define     ECI_EI_TRACE                0x4
*/
/* throw the following line away in non-development mode */
  rc = rxfuncdrop('ECI')

return 0


/*****************************************************************/
/*           some utililty fuunctions                            */
/*****************************************************************/

check_rc: procedure  expose ECI.
arg commarea

  msgfile = "rxeci.msg"
  rc = show_eci_stem()
  if (ECI.return_code = 0) then do
     /* code to process commarea2 goes here */
     say "returned commarea:"
     say commarea
     end
  else do
     say "RxECI failed"
     do until lines(msgfile) = 0
        line = linein(msgfile)
        parse var line returncode message
        if ECI.return_code = returncode then say message
        end
     end
     rc = lineout(msgfile)
return(0)

show_eci_stem: procedure  expose ECI.

say "Contents of ECI stem:"
say "      call_type         : " ECI.call_type
say "      program name      : " ECI.program_name
say "      userid            : " ECI.userid
say "      password          : " ECI.password
say "      transid           : " ECI.transid
say "      abend_code        : " ECI.abend_code
say "      commarea_length   : " ECI.commarea_length
say "      timeout           : " ECI.timeout
say "      sys_return_code   : " ECI.sys_return_code
say "      extended_mode     : " ECI.extend_mode
say "      sem_handle        : " ECI.ASYNC_NOTIFY.SEM_HANDLE
say "      luw_token         : " ECI.luw_token
say "      return code       : " ECI.return_code
return(0)

/* return code from ECI */
  ECI.return_code       = ""                      /* respone  */

FAILURE:
  Say 'Failure to add function at line' sigl
  Address CMD "PAUSE"
  Return 1
