/* RXEPI3.CMD  */
/*******************************************************************/
/*                                                                 */
/* MODULE NAME      RXEPI3.CMD                                     */
/*                                                                 */
/* DESCRIPTIVE NAME Demonstrates RxEPI                             */
/*                                                                 */
/* Statement:       Licensed Materials - Property of IBM           */
/*                                                                 */
/*                  CA31 SupportPac                                */
/*                  (c) Copyright IBM Corp. 1995.                  */
/*                                                                 */
/*                  See Copyright Instructions.                    */
/*                                                                 */
/*                  All rights reserved.                           */
/*                                                                 */
/*                  U.S. Government Users Restricted Rights - use, */
/*                  duplication or disclosure restricted by GSA    */
/*                  ADP Schedule Contract with IBM Corp.           */
/*                                                                 */
/* Status:          Version 1 Release 4 Mod 3                      */
/* Modified         July 3rd 1995                                  */
/*                  Corrections by Steve Longhurst                 */
/* Modified         Sept 6th 1995                                  */
/*                  Corrections by Bill Matthews                   */
/*                                                                 */
/*  NOTES :-                                                       */
/*    DEPENDENCIES = OS/2 V2.x,CICS OS/2, RxEPI                    */
/*                   None                                          */
/*    RESTRICTIONS = none                                          */
/*    MODULE TYPE  = Command file                                  */
/*    PROCESSOR    = PS/2 and PC                                   */
/*                                                                 */
/*******************************************************************/
/*         Program is a RXEPI based REXX 3270 Terminal emulator    */
/*                 With debugging code                             */
/*******************************************************************/
SIGNAL ON FAILURE  NAME FAILURE
SIGNAL ON ERROR    NAME FAILURE
SIGNAL ON NOTREADY NAME FAILURE
/*SIGNAL ON NOVALUE NAME FAILURE*/
SIGNAL ON SYNTAX  NAME FAILURE
trace off

  Call RexxSetup                          /* Ensure RexxUtil loaded*/

  logfile = ''
  logfile = 'NULL'
  logfile = 'rxepi.log'
/*  'rm rxepi.log'*/

/*
   The Field stem is used to hold the 3270 data stream in a more
   structured way.
   Field.0 is the number of fields
*/

Field.0 = 0

/* install Default Terminal at CICS OS/2 */
TERMNAME = ''
TERMTYPE = ''
TERMINDEX = Init_EPI()

/* show a bit of the now available information */
say "CICS Name:" LIST._SYSTEMNAME
say "TermIndex:" TERMINDEX
/*Next line removed for compatibility with CICS Clients - JH 6 Sep 95 */
/*say "Termname" DETAILS._TERMNAME "Netname" DETAILS._NETNAME */
say "MaxData" DETAILS._MAXDATA

/*******************************************************************/
/* start transaction                                               */
/*******************************************************************/

say "Enter Transaction Code ?"
pull TRANSAKTION
if length(TRANSAKTION) < 4 then
  TRANSAKTION = substr(TRANSAKTION, 1, 4, ' ')

DATA = '27'x || '20'x || '20'x
DATA = DATA || TRANSAKTION
lrc = to_stderr(logfile,DATA)
DATASIZE = LENGTH(DATA)
lrc = to_stderr(logfile,DATASIZE)

rc = RxEpiStartTran(TERMINDEX,'', DATA, DATASIZE)
if rc \= 'CICS_EPI_NORMAL' then do
        rc = ErrorExit("RxEpiStartTran", rc, TERMINDEX);
        end

/*******************************************************************/
/* event processing loop                                           */
/*******************************************************************/

do forever
lrc = to_stderr(logfile,"Getting Event")
    /* first we have to make some space to hold the data in */
    EVENT._SIZE = DETAILS._MAXDATA

    rc = RxEpiGetEvent(TERMINDEX,'CICS_EPI_WAIT','EVENT.')
    lrc = to_stderr(logfile, "rc =" rc)
    lrc = to_stderr(logfile, "TERMINDEX"  EVENT._TERMINDEX)
    lrc = to_stderr(logfile, "EVENT    "  EVENT._EVENT)
    lrc = to_stderr(logfile, "ENDREASON"  EVENT._ENDREASON)
    lrc = to_stderr(logfile, "TRANSID  "  EVENT._TRANSID)
    lrc = to_stderr(logfile, "ABENDCODE"  EVENT._ABENDCODE)
    lrc = to_stderr(logfile, "SIZE     "  EVENT._SIZE)

    if rc \= 'CICS_EPI_NORMAL' & rc \= 'CICS_EPI_ERR_MORE_EVENTS' then do
        rc = ErrorExit("RxEpiGetEvent", rc, TERMINDEX)
        end

    Select
        when EVENT._EVENT = "CICS_EPI_EVENT_DUMMY" then do
             lrc = to_stderr(logfile,"handling CICS_EPI_EVENT_DUMMY")
             end
        when EVENT._EVENT = "CICS_EPI_EVENT_SEND" then do
             lrc = to_stderr(logfile,"handling CICS_EPI_EVENT_SEND")
             /* new data has arrived, so previous is to be dropped */
             drop Field.
             /* map data stream onto stem */
             COMMAND = data2field(EVENT._DATA)
             select
                when COMMAND = '31'x then do
                     index = field2buffer()
                     end
                when COMMAND = '35'x then do
                     buffer  = ''
                     index = field2buffer()
                     end
                otherwise
                     say 'COMMAND =' COMMAND
                end
             end
        when EVENT._EVENT = "CICS_EPI_EVENT_CONVERSE" then do
             lrc = to_stderr(logfile,"handling CICS_EPI_EVENT_CONVERSE")
             /* CICS wants some response */
             DATA = GetAIDKey(index)
             lrc = to_stderr(logfile, DATA)
             DATASIZE = LENGTH(DATA)
             lrc = to_stderr(logfile, DATASIZE)
             /* send reply back */
             rc = RxEpiReply(TERMINDEX, DATA, DATASIZE)
             lrc = to_stderr(logfile,"rc from reply" rc)
             if rc \= 'CICS_EPI_NORMAL' then do
                rc = ErrorExit("RxEpiReply", rc, TERMINDEX);
                end
             end
        when EVENT._EVENT = "CICS_EPI_EVENT_END_TRAN" then do
             lrc = to_stderr(logfile,"handling CICS_EPI_EVENT_END_TRAN")
             /* transaction has ended, if there is no one waiting, ask for
                next one */
             if EVENT._TRANSID = '' then do
                say ''
                say "Enter Transaction Code?"
                pull TRANSAKTION
                if length(TRANSAKTION) < 4 then
                  TRANSAKTION = substr(TRANSAKTION, 1, 4, ' ')

                /* EXIT terminates terminal */
                if (TRANSAKTION = 'EXIT') then do
                    lrc = to_stderr(logfile,"leaving first loop")
                    leave
                    end
                else do
                    /* start new one */
                    DATA = '27'x || '20'x || '20'x
                    DATA = DATA || TRANSAKTION
                    lrc = to_stderr(logfile,DATA)
                    DATASIZE = LENGTH(DATA)
                    lrc = to_stderr(logfile,DATASIZE)

                    rc = RxEpiStartTran(TERMINDEX,'', DATA, DATASIZE)
                    if rc \= 'CICS_EPI_NORMAL' then do
                       rc = ErrorExit("RxEpiStartTran", rc, TERMINDEX);
                       end
                    end
                end
             else do
                /* next transaktion is already waiting
                   CICS waits for some input  */
                lrc = to_stderr(logfile, "transaction waiting")
                DATA = GetAIDKey(index)
                lrc = to_stderr(logfile, DATA)
                DATASIZE = LENGTH(DATA)
                lrc = to_stderr(logfile, DATASIZE)
                rc = RxEpiStartTran(TERMINDEX,'', DATA, DATASIZE)
                if rc \= 'CICS_EPI_NORMAL' then do
                   rc = ErrorExit("RxEpiStartTran", rc, TERMINDEX);
                   end
                end
             end
        when EVENT._EVENT = "CICS_EPI_EVENT_START_ATI" then do
             lrc = to_stderr(logfile, "handling CICS_EPI_EVENT_START_ATI")
        end
        when EVENT._EVENT = "CICS_EPI_EVENT_END_TERM" then do
             lrc = to_stderr(logfile, "handling CICS_EPI_EVENT_END_TERM")
             leave
        end
        otherwise lrc = to_stderr(logfile, "Unknown Event")
    end
end
rc = End_EPI(TERMINDEX)

return 0

/*****************************************************************/
/*****************************************************************/
/*****************************************************************/
/*                 Some utility functions                        */
/*****************************************************************/
/*****************************************************************/
/*****************************************************************/


 /*********************************************************************/
 /* data2field() does the time comsuming tasks of examining the 3270  */
 /* data stream and maps it onto stem Field.                          */
 /* Good canditate to code in 'C' to speed up processing.             */
 /*                                                                   */
 /*********************************************************************/
data2field: procedure expose logfile DETAILS. Field. binvalues dispvalues
arg DATA

  lrc = to_stderr(logfile,"DATA")
  lrc = to_stderr(logfile,DATA)

/* analyse data stream */
index = 1
i = 1

/* Command */
   Field.index.Content = substr(DATA, i, 1)
        Field.index.Type    = 'CMD'
   index = index  + 1
   i = i + 1

/* Write Control Character */
   Field.index.Content = substr(DATA, i, 1)
        Field.index.Type    = 'WCC'
   i = i + 1

/* loop through data */
   datalength = length(DATA) + 1
   do while ( i < datalength)
      data_type = ''
      ORDER      = substr(DATA, i, 1)
      Select
         when ORDER = '1D'X then do     /* CICS_EPI_ORDER_SF Start Field */
             length = 2
             data_type = 'SF'
             end

         when ORDER = '10'X then do     /* CICS_EPI_ORDER_SFE Start Field Extended */
             length = 2 + 2 * c2d(substr(DATA, i + 1, 1))
             data_type = 'SFE'
             end

         when ORDER = '11'X then do     /* CICS_EPI_ORDER_SBA Set Buffer Address */
             length = 3
             data_type = 'SBA'
             end

         when ORDER = '13'X then do     /* CICS_EPI_ORDER_IC  Insert Cursor */
             length = 1
             data_type = 'IC'
             end

          otherwise
/* CICS_EPI_ORDER_SA  0x1F Set Attribute (Not a standard 3270 value)    */
/* CICS_EPI_ORDER_MF  0x1E Modify Field (Not a standard 3270 value)     */
/* CICS_EPI_ORDER_PT  0x09 Program Tab */
/* CICS_EPI_ORDER_RA  0x14 Repeat to Address */
/* CICS_EPI_ORDER_EUA 0x12 Erase Unprotected to Address */
             length = 1
             data_type = 'D'
        end /* select */

   if (data_type = 'D') then do         /* build up data field */
        Field.index.Content = Field.index.Content || ORDER
        Field.index.Type    = data_type
        end
   else do                              /* start next field */
        if (Field.index.Content \= "") then index = index + 1
        Field.index.Content = substr(DATA, i, length)
        Field.index.Type    = data_type
        index = index + 1
        Field.index.Content = ""
        end
   i = i + length
   end /* do */

Field.0 = index
return(Field.1.Content)

 /*********************************************************************/
 /* field2buffer() analyses the field stem and puts displayable data  */
 /* in a display buffer. In addition a few more variables are stored  */
 /* in the stem to speed up later processing.                         */
 /*                                                                   */
 /*********************************************************************/
field2buffer: procedure expose logfile DETAILS. Field. binvalues dispvalues buffer

   Call SysCls
   /* first position in a REXX variable is 1, not 0 */
   cursor = 1
   startcursor = 1
   /* display screen */
   index = 1
   Field.inputfield.0 = 0
   do while ( index < Field.0 + 1) /* go through fields */
        Select
            when Field.index.Type = 'CMD' then do
                Field.index.dez = cursor
                end

            when Field.index.Type = 'IC' then do
                /* this is where the cursor blinks for the first input */
                startcursor = cursor - 1
                end

            when Field.index.Type = 'WCC' then do
                Field.index.dez = cursor
                end

            when Field.index.Type = 'D' then do
                Field.index.dez = cursor
                buffer = overlay(Field.index.Content,buffer,cursor)

                /* check if CICS has send some data to input fields
                   if so, declare them as modified and send back later */
                savecursor = cursor
                saveindex  = index
                /* we have to find the attribute field */
                do while ( cursor > 0)
                   if (DATATYPE(Field.Bufferaddress.cursor,'N') = 1) then do
                        index = Field.Bufferaddress.cursor
                        leave
                        end
                   cursor = cursor - 1
                   lrc = to_stderr(logfile, "cursor" cursor)
                   end
                lrc = to_stderr(logfile, "cursor" cursor)
                lrc = to_stderr(logfile, "index" index)
                lrc = to_stderr(logfile, "dez" Field.index.dez)
                lrc = to_stderr(logfile, "indexdata " Field.index.Content)
                /* if it was a input field, mark is as modified */
                if (Field.index.protect = '0') then do
                    Field.index.modify   = '1'
                    end
                cursor = savecursor
                index  = saveindex
                cursor =  cursor + length(Field.index.Content)
                end

            when Field.index.Type = 'SBA' then do
                /* next address in buffer */
                BA = substr(Field.index.Content, 2, 2)
                cursor = BA2dez(BA) + 1
                Field.index.dez = cursor
                end

            when Field.index.Type = 'SFE' then do
                /* tedious task of attrubte processing */
                PAIRS = c2d(substr(Field.index.Content, 2, 1))
                Field.index.0        = PAIRS
                k = 1
                i = 3
                do while ( k < PAIRS+1)
                  EAType = c2x(substr(Field.index.Content, i, 1))
                  i = i + 1
                  EAValue = substr(Field.index.Content, i, 1)
                  i = i + 1
                  Field.index.k.Type  = EAType

                  Select
                     when EAType = '42' then do
                          Field.index.k.Value = EAValue
                          end
                     when EAType = '43' then do
                          end
                     when EAType = '41' then do
                          end
                     when EAType = 'C0' then do
                          /* 3270 attributes */
                          Field.index.k.Value = c2x(translate(EAValue,binvalues,dispvalues))
                          ATTRbin  = x2b(Field.index.k.Value)
                          Field.index.protect  = substr(ATTRbin, 3, 1)
                          if (Field.index.protect  = '0') then do
                                Field.inputfield.0 = Field.inputfield.0 + 1
                                nofield = Field.inputfield.0
                                Field.inputfield.nofield = cursor
                                end
                          Field.index.alpha    = substr(ATTRbin, 4, 1)
                          Field.index.display0 = substr(ATTRbin, 5, 1)
                          Field.index.display1 = substr(ATTRbin, 6, 1)
                          if (Field.index.display0 = '1' ) & (Field.index.display1 = '1') then
                                  Field.index.hide  = '1'
                          Field.index.reserved = substr(ATTRbin, 7, 1)
                          if (substr(ATTRbin, 8, 1) = '1') then
                                  Field.index.modify   = '0'
                          end
                     when EAType = 'C1' then do
                          end
                     when EAType = 'C2' then do
                          end
                     when EAType = '00' then do
                          end
                     otherwise
                     end
                  k = k + 1
                  end
                Field.index.dez = cursor
                /* this helps to find the field, which sits at a
                   certain bufferaddress */
                Field.Bufferaddress.cursor = index
                buffer = overlay(' ',buffer,cursor)
                cursor = cursor + 1
                end

            otherwise
            end
        index = index + 1
        end

/* now display */
rc = charout(,buffer)
rc = SetCursor(startcursor)

/* log to file */
lrc = to_stderr(logfile, "ALL  ")
   index = 1
   do while ( index < Field.0 + 1)
        lrc = to_stderr(logfile, "index  " index)
        lrc = to_stderr(logfile, "type   " Field.index.Type)
        lrc = to_stderr(logfile, "length " length(Field.index.Content))
        lrc = to_stderr(logfile, "content" Field.index.Content)
        lrc = to_stderr(logfile, "content" c2x(Field.index.Content))
        index = index + 1
        end

return(index)

 /*********************************************************************/
 /* ErrorExit() - Report errors and exit                              */
 /*********************************************************************/
ErrorExit: procedure
arg function, rc, TERMINDEX

   say function || "() call failed "
   if (rc \= "CICS_EPI_ERR_FAILED") then do
      say "with return code message:" rc
      end
   else do
                                       /* EPI_ERR_FAILED, more error  */
                                       /* information in GetSysError  */
      rc = RxEpiGetSysError(TermIndex, 'SYSERR.');
      say "with a system error"
      say "Cause:" SYSERR._Cause
      say "Error:" SYSERR._Value
      say "Message:" SYSERR._Msg
      end
rc = RxEpiTerminate()
exit
return (0)

 /*********************************************************************/
 /* Failure()                                                         */
 /*********************************************************************/
FAILURE:
  Say 'Failure line' sigl
  Say '+++' sourceline(sigl)
  rc = RxEpiTerminate()
  exit

 /*********************************************************************/
 /* Init_EPI()                                                        */
 /*********************************************************************/
 /*Updated next 2 lines - JH 6 Sep 95 */
Init_EPI: procedure expose LIST. DETAILS. binvalues dispvalues buffer,
          TERMNAME TERMTYPE

/* add the Epi functionality as external function */
    if rxfuncquery('RxEpiLoad') Then Do
    call RxFuncDrop 'RxEpiLoad'
    call RxFuncAdd 'RxEpiLoad', 'RXEPI', 'RxEpiLoad'
    call RxEpiLoad
    Say 'RxEpiLoad is complete.'
    end

/* empty character buffer */
buffer = ''
/* tranlation tables */
binvalues  =              '00 01 02 03 04 05 06 07 08 09 0A 0B 0C 0D 0E 0F'X
dispvalues =              '20 41 42 43 44 45 46 47 48 49 5B 2E 3C 28 2B 21'X
binvalues  = binvalues  ||'10 11 12 13 14 15 16 17 18 19 1A 1B 1C 1D 1E 1F'X
dispvalues = dispvalues ||'26 4A 4B 4C 4D 4E 4F 50 51 52 5D 24 2A 29 3B 5E'X
binvalues  = binvalues  ||'20 21 22 23 24 25 26 27 28 29 2A 2B 2C 2D 2E 2F'X
dispvalues = dispvalues ||'2D 2F 53 54 55 56 57 58 59 5A 7C 2C 25 5F 3E 3F'X
binvalues  = binvalues  ||'30 31 32 33 34 35 36 37 38 39 3A 3B 3C 3D 3E 3F'X
dispvalues = dispvalues ||'30 31 32 33 34 35 36 37 38 39 3A 23 40 27 3D 22'X

/* initialize  EPI */
rc = RxEpiInitialize()
if rc \= 'CICS_EPI_NORMAL' then do
        rc = ErrorExit("RxEpiInitialize", rc, TERMINDEX);
        end
/* list available systems */
rc = RxEpiListSystems('','SYSTEMS','LIST.')
if rc \= 'CICS_EPI_NORMAL' then do
        rc = ErrorExit("RxEpiListSystems", rc, TERMINDEX);
        end
/* say "Number of Systems:" SYSTEMS*/

/* add a terminal */
rc = RxEpiAddTerminal('',LIST._SYSTEMNAME,TERMNAME,TERMTYPE,'','DETAILS.','TERMINDEX')
if rc \= 'CICS_EPI_NORMAL' then do
        rc = ErrorExit("RxEpiAddTerminal", rc, TERMINDEX);
        end
return(TERMINDEX)

 /*********************************************************************/
 /* End_EPI()                                                         */
 /*********************************************************************/
End_EPI: procedure expose LIST. DETAILS. Field. buffer
parse arg TERMINDEX

/* Delete Terminal */
rc = RxEpiDelTerminal(TERMINDEX)
if rc \= 'CICS_EPI_NORMAL' then do
    rc = ErrorExit("RxEpiDelTerminal", rc, TERMINDEX);
    end

/* event processing */
do forever
    EVENT._SIZE = 500
    rc = RxEpiGetEvent(TERMINDEX,'CICS_EPI_WAIT','EVENT.')
    if rc \= 'CICS_EPI_NORMAL' & rc \= 'CICS_EPI_ERR_MORE_EVENTS' then do
        rc = ErrorExit("RxEpiGetEvent", rc, TERMINDEX)
        end
    Select
        when EVENT._EVENT = "CICS_EPI_EVENT_END_TERM" then do
             say "Terminal ended"
             leave
        end
        otherwise say "Getting next event"
    end
end
/* terminating */
rc = RxEpiTerminate()
if rc \= 'CICS_EPI_NORMAL' then do
        rc = ErrorExit("RxEpiTerminate", rc, TERMINDEX);
        end
/* dropping functions, can be omitted */
Call RxEpiDrop
Say 'The functions have now been dropped.'

return (rc)

 /*********************************************************************/
 /* RexxSetup - Ensure RexxSetup is loaded                            */
 /*********************************************************************/
RexxSetup: Procedure
  Call RxFuncAdd 'SysLoadFuncs', 'RexxUtil', 'SysLoadFuncs'
  Call SysLoadFuncs
Return 0

 /*********************************************************************/
 /* to_stderr                                                         */
 /*********************************************************************/
to_stderr: Procedure
parse arg filename, string
  Select
        when filename = '' then do
             say string
        end
        when filename = 'NULL' then do
        end
        otherwise rc = lineout(filename, string)
      end
Return 0

 /*********************************************************************/
 /*GetAIDKEy() has the wonderful task of processing the user input    */
 /*********************************************************************/
GetAIDKey: Procedure expose logfile Field. binvalues dispvalues DETAILS. buffer
parse arg index

AIDKEY = '0'x
do while (AIDKEY = '0'x)
    cursor = GetCursor()
    char = SysGetKey(NOECHO)
    key = c2d(char)
    lrc = to_stderr(logfile,"Key" key)
    if key = '0' | Key = '224' then do
       key = key || c2d(SysGetKey(NOECHO))
       lrc = to_stderr(logfile,"Key " key)
       end

    Select
        when key = '9' then do  /* TAB */
             maxnofield = Field.inputfield.0
             nofield = 1
             lrc = to_stderr(logfile, "maxnofield" maxnofield)
             cursor = GetCursor()
             if (maxnofield > 0) then
                do forever
                    /* find next input field */
                    lrc = to_stderr(logfile, "i.cursor" cursor)
                    lrc = to_stderr(logfile, "i.nofield" Field.inputfield.nofield)
                    lrc = to_stderr(logfile, "nofield" nofield)
                    if (cursor <= Field.inputfield.nofield) then do
                        lrc = to_stderr(logfile, "setting cursor")
                        rc = SetCursor(Field.inputfield.nofield)
                        leave
                        end
                    nofield = nofield + 1
                    if (nofield > maxnofield ) then do
                        nofield = 1
                        cursor = 1
                        end
                    end
        end
        when key == '22472' then do  /* up */
             cursor = GetCursor() - 81
             rc = SetCursor(cursor)
        end
        when key == '22480' then do  /* down */
             cursor = GetCursor() + 79
             rc = SetCursor(cursor)
        end
        when key == '22475' then do  /* left */
             cursor = GetCursor() - 2
             rc = SetCursor(cursor)
        end
        when key == '22477' then do  /* right */
             cursor = GetCursor()
             rc = SetCursor(cursor)
        end
        when key == '059' then do  /* F1*/
             AIDKEY = '31'x
        end
        when key == '060' then do  /* F2 */
             AIDKEY = '32'x
        end
        when key == '061' then do  /* F3 */
             AIDKEY = '33'x
        end
        when key == '062' then do  /* F4 */
             AIDKEY = '34'x
        end
        when key == '063' then do  /* F5 */
             AIDKEY = '35'x
        end
        when key == '064' then do  /* F6 */
             AIDKEY = '36'x
        end
        when key == '065' then do  /* F7 */
             AIDKEY = '37'x
        end
        when key == '066' then do  /* F8 */
             AIDKEY = '38'x
        end
        when key == '067' then do  /* F9 */
             AIDKEY = '39'x
        end
        when key == '068' then do  /* F10 */
             AIDKEY = '3A'x
        end
        when key == '0133' then do  /* F11 */
             AIDKEY = '23'x
        end
        when key == '0134' then do  /* F12 */
             AIDKEY = '40'x
        end
        when key == '13' then do   /* Enter */
             AIDKEY = '27'x
        end
        otherwise
             /* just a normal key */
             cursor = GetCursor()

             /* find attribute field */
             do while ( cursor > 0)
                if (DATATYPE(Field.Bufferaddress.cursor,'N') = 1) then do
                        index = Field.Bufferaddress.cursor
                        leave
                        end
                cursor = cursor - 1
                lrc = to_stderr(logfile, "cursor" cursor)
                end
             cursor = GetCursor()
             lrc = to_stderr(logfile, "cursor" cursor)
             lrc = to_stderr(logfile, "index" index)
             lrc = to_stderr(logfile, "dez" Field.index.dez)
             lrc = to_stderr(logfile, "indexdata " Field.index.Content)

             /* if input field, then mark as modified ... */
             if (Field.index.protect = '0') then do
                 Field.index.modify   = '1'
                 lrc = to_stderr(logfile, "cursor" cursor, char)
                 buffer = overlay(char,buffer,cursor,1,' ')
                 lrc = to_stderr(logfile, "buffer" buffer)

                 /* .. and show character if not hidden */
                 if (Field.index.hide \= '1') then
                      rc  = charout(,char)
                 else
                      rc  = charout(,' ')
                 end
             else do
                 /* this user tried to enter data in a protected field
                    beep at him, but mildly */
                 rc = beep(100,20)
                 end
    end
end

/* prepare data */
data = AIDKEY || right(dez2BA(cursor),2)

   maxcursor = DETAILS._NUMCOLUMNS * DETAILS._NUMLINES + 1
   lrc = to_stderr(logfile, "maxcursor" maxcursor)
   /* find first input field */
   maxnofield = Field.inputfield.0
   lrc = to_stderr(logfile, "maxnofield" maxnofield)
   nofield = 1
   if (maxnofield > 0) then
   do while ( nofield < maxnofield + 1)

        /* take all the input fields and append */
        lrc = to_stderr(logfile, "nofield" nofield)
        cursor = Field.inputfield.nofield
        index  = Field.Bufferaddress.cursor
        if (Field.index.modify = '1') then do
            lrc = to_stderr(logfile, "index cursor" index cursor)
            startcursor = cursor
            cursor = cursor + 1
            /* find next field */
             do while ( cursor < maxcursor)
                if (DATATYPE(Field.Bufferaddress.cursor,'N') = 1) then do
                        leave
                        end
                lrc = to_stderr(logfile, "cursor" cursor)
                cursor = cursor + 1
                end
            fieldlength = cursor - startcursor - 1
            lrc = to_stderr(logfile, "fieldlength" fieldlength)
            fielddata = substr(buffer,startcursor + 1,fieldlength)
            lrc = to_stderr(logfile, "->" fielddata)
            BA = right(dez2BA(startcursor),2)
            data = data || '11'x || BA || fielddata
            lrc = to_stderr(logfile, "data" data)
            lrc = to_stderr(logfile, "data" c2x(data))
            lrc = to_stderr(logfile, "modify  " Field.index.modify)
            end

        nofield = nofield + 1
        end

return(data)


 /*********************************************************************/
 /* GetColor()                                                        */
 /*********************************************************************/
GetColor: Procedure expose logfile
parse arg Colorvalue

    Select
        when Colorvalue = '00'x then do
             COLOR = "DFHDFCOL"
             end
        when Colorvalue = '1' then do
             COLOR = "DFHBLUE"
             end
        when Colorvalue = '2' then do
             COLOR = "DFHRED"
             end
        when Colorvalue = '3' then do
             COLOR = "DFHPINK"
             end
        when Colorvalue = '4' then do
             COLOR = "DFHGREEN"
             end
        when Colorvalue = '5' then do
             COLOR = "DFHTURQ"
             end
        when Colorvalue = '6' then do
             COLOR = "DFHYELLO"
             end
        when Colorvalue = '7' then do
             COLOR = "DFHNEUTR"
             end
        otherwise COLOR = "Unknown COLOR"
  end
return(COLOR)

 /*********************************************************************/
 /* BA2dez() translates buffer address to decimal, there are probably */
 /* more efficinet ways to do this, but it works.                     */
 /*********************************************************************/
BA2dez: Procedure expose binvalues dispvalues DETAILS. logfile
parse arg BA
     BAhex  = c2x(translate(BA,binvalues,dispvalues))
     lrc = to_stderr(logfile, BA c2x(BA) BAhex x2b(BAhex))
     BAlow  = substr(BAhex, 3, 2)
     BAhigh = substr(BAhex, 1, 2)
     lrc = to_stderr(logfile, x2b(BAlow) x2b(BAhigh))
     BAdez = x2d(BAlow)+x2d(BAhigh)*64
     lrc = to_stderr(logfile, BAdez)
return (BAdez)

 /*********************************************************************/
 /* dez2BA() translates decimal back to buffer address, see BA2dez()  */
 /*********************************************************************/
dez2BA: Procedure expose binvalues dispvalues DETAILS. logfile
parse arg BAdez

     lrc    = to_stderr(logfile,"dez2BA" BAdez BAhigh BAlow)
     BAhigh = BAdez%64
     BAlow  = BAdez//64
     lrc    = to_stderr(logfile,"dez2BA" BAdez BAhigh BAlow)
     BAhex  = d2x(BAhigh) || d2x(BAlow)
     lrc    = to_stderr(logfile,"dez2BA" BAhex BA)
     BA     = translate(x2c(BAhex),dispvalues, binvalues)
     lrc    = to_stderr(logfile,"dez2BA" BAhex BA)
return (BA)

 /*********************************************************************/
 /* SetCursor() takes a decimal cursor address.                       */
 /*********************************************************************/
SetCursor: Procedure expose DETAILS.
parse arg cursor
      row = cursor%DETAILS._NUMCOLUMNS
      col = cursor//DETAILS._NUMCOLUMNS
      rc = SysCurPos(row,col)
return (rc)

 /*********************************************************************/
 /* GetCursor() returns a decimal cursor address, we are not inter-   */
 /* ested in rows an columns n this program.                          */
 /*********************************************************************/
GetCursor: Procedure expose DETAILS.
      parse value SysCurPos() with row col
      cursor = col + row * DETAILS._NUMCOLUMNS + 1
return (cursor)
