/* HECHO.CMD   */
/*******************************************************************/
/*                                                                 */
/* MODULE NAME      HECHO.CMD                                      */
/*                                                                 */
/* DESCRIPTIVE NAME Use RxECI to the ECHO application on another   */
/*                  CICS system connected to CICS OS/2.            */
/*                                                                 */
/* Statement:       Licensed Materials - Property of IBM           */
/*                                                                 */
/*                  CA31 SupportPac                                */
/*                  (c) Copyright IBM Corp. 1988, 1994.            */
/*                                                                 */
/*                  See Copyright Instructions.                    */
/*                                                                 */
/*                  All rights reserved.                           */
/*                                                                 */
/*                  U.S. Government Users Restricted Rights - use, */
/*                  duplication or disclosure restricted by GSA    */
/*                  ADP Schedule Contract with IBM Corp.           */
/*                                                                 */
/* Status:          Version 1 Release 1                            */
/* Modified         19th May 1994                                  */              
/*                  Mean calculation modified to reduce overhead   */
/*                                                                 */
/*  NOTES :-                                                       */
/*    DEPENDENCIES = OS/2 2.x, CICS OS/2, RxECI                    */
/*                   None                                          */
/*    RESTRICTIONS = none                                          */
/*    MODULE TYPE  = Command file                                  */
/*    PROCESSOR    = PS/2 and PC                                   */
/*                                                                 */
/*******************************************************************/
/*                                                                 */
/*  HECHO - This is the front-end for the durability - echo test   */
/*          Where target is on the host CICS/ESA system            */
/*                                                                 */
/*  Please see the prolog for the TESTRUN.CMD for additional       */
/*  information.                                                   */
/*                                                                 */
/*  Since the target of this request is another CICS system, then  */
/*  the CICS OS/2 PPT must have a PPT defined for a Remote         */
/*  program:                                                       */
/*                                                                 */
/*      Program Name = ECHOHOST                                    */
/*                                                                 */
/*        SYSID      = xxxx                                        */
/*    Remote Program = ECHO                                        */
/*   Resident/Remote = R                                           */
/*                                                                 */
/*  The ECHO source should be sent (uploaded) to the target,       */
/*  translated, compiled, linked and defined to that system.       */
/*  It has been used on both MVS and VSE systems.                  */
/*                                                                 */
/*  The design of this command is to make repeated calls to the    */
/*  ECHO program with a variable COMMAREA size, ranging from       */
/*  50 to 3500 bytes, in increments of 50 bytes.  The upper bound  */
/*  on this may be changed to something smaller by modifying the   */
/*  MAXLENGTH value shown below.                                   */
/*                                                                 */
/*  The ECHO program will return a COMMAREA equal to the le+gth    */
/*  of the COMMAREA coming from this command -- and -- will        */
/*  force the commaree to be filled with blanks (spaces).          */
/*  Source for ECHO is also provided.                              */
/*                                                                 */
/*  It also uses a random amount of wait time, between 1 and       */
/*  10 seconds.  The upper bound on this is defined by the         */
/*  MAXSLEEP value shown below.                                    */
/*                                                                 */
/*  A DFHCVT is required when connecting to mainframe CICS.        */
/*  A sample, DFHCVT.TAB is provided.                              */
/*                                                                 */
/*-----------------------------------------------------------------*/
DEBUG = 0
  MAXLENGTH = 3500                      /* Upper limit on commarea */
  MAXSLEEP  = 10                        /* Max sleep time          */
  Arg log_file .                        /* Name of the Log File    */
  Call RexxSetup                        /* Ensure RexxUtil loaded  */
  Call EciSetup                         /* Ensure RxCICS loaded    */
  Parse Value '0 0 0 0 0 0' With mean_count high i bs t log log_id response_list
  If log_file<>'' Then
    Do
      logname=filespec('name',log_file)
      logname=substr(logname,1,lastpos('.',logname)-1)
      log_id=left(logname,8)
      log=1
      Say 'logging to:' log_file
    End
  call SysCls                                 /* clear the screen        */
  ECI.program_name      = "ECHOHOST"          /* required                */
  sleep=0                                     /* other parms will default*/
  started=time() date() date('D')
  say 'Test started at' started 'for a maximum' MAXLENGTH 'byte commarea'
  if Log Then '@ECHO' log_id 'started at' started '>>' log_file
  do Forever
    Parse Value Select(MAXLENGTH) with CALI  /* Compute length            */
    t=t+1                                    /* Count number of ECI calls */
    commareaR=left('MyEchoTest',CALI)        /* input for ECI call        */
    ECI.commarea_length = CALI               /* set the length            */
    a=time('R')                              /* start the timer           */
    commarea2 = ECI('ECI.',commareaR)        /* make the call             */
    b=time('E')                              /* stop the timer            */
    CALO=length(commarea2)                   /* Commarea length Returned  */
    if b>high then high=b
    bs=bs+b                                  /* add to the total time     */
    call SysCurPos '1', '1'                  /* reposition the screen     */
    say 'ECI Count='right(t,5)               /* make a report             */
    say left(commarea2,50) '(len=' length(commarea2)')    '
    call ecirc                               /* Check the ECI return code */
    sleep=random(1,MAXSLEEP)                 /* random sleep time         */
    call SysSleep sleep                      /* sleep some time           */
    If eci.return_code<>0 Then Leave         /* Run till non-zero         */
  end
  If Log Then
    Do
      if eci.abend_code<>'' Then '@ECHO' log_id eci.abend_code
      '@ECHO' log_id 'stoped at' time() date() date('D') '>>' log_file
    End
  '@EXIT'
  exit 999

FAILURE:
  Say 'Failure to add function at line' sigl
  Return 1
/*  Check the ECI return code ----------------------------------------    */
ecirc:
    avg=bs/t
    avg=format(avg,2,3)
    last=format(b,2,3)
    b=format(b,2,3)
    meant=mean(b)
    high=format(high,2,3)
    say 'Avg ='avg,                      /* report average time      */
        'Last='last,                    /* and last time            */
        'mean='meant,
        'sleep=' sleep,
        'longest=' high
    If Log Then
      Do
         log_rec=log_id Left(eci.program_name,8),
         right(t,8) left(avg,6) left(b,6) left(meant,6) right(sleep,2),
         left(high,6) right(CALI,4) right(eci.return_code,3),
         time() date() eci.abend_code
         '@ECHO' log_rec '>>' log_file
      End
    If eci.return_code ='0'   Then            /* if good, goback          */
      Do
         say 'rc='eci.return_code 'ECI-NO-ERROR'
         Return
      End
    CALL CHECKECI eci.return_code

Return
/*-------------------------------------------------------*/
/* Select - Generate a random size value in increments   */
/*          of 50 bytes  - range 50 to 32000             */
/*-------------------------------------------------------*/
Select: Procedure
  Arg MAXLENGTH                     /*Get Max set by user*/
  scale=50                          /* Increments        */
  upper = MAXLENGTH/scale           /* Upper bounds      */
  cmsize=(random(1,upper) * (scale))/* Compute size      */
Return cmsize                       /* return value      */
/*-------------------------------------------------------*/
/*------------------------------------------------------------------------*/
/* Compute the mean response time for 500 values only to minimize overhead*/
/*------------------------------------------------------------------------*/
Mean: Procedure Expose response_list mean_count
  Arg resp .                                 /*                           */
  new_list=''                                /*                           */
  mean_count=mean_count+1                    /* count the number of values*/
  If mean_count>499 Then                     /* use up to 500 values      */
    Do                                       /* to kee[ down overhead     */
      mean_count=0                           /* reset the count           */
      response_list=''                       /* and the list              */
    End                                      /*                           */
  If response_list='' Then                   /*                           */
    Do                                       /*                           */
      response_list=resp                     /*                           */
      Return resp                            /*                           */
    End                                      /*                           */
  Count=words(response_list)                 /*                           */
  first_word=word(response_list,1)           /*                           */
  last_word =word(response_list,Count)       /*                           */
  If resp<first_word Then                    /*                           */
    Do                                       /*                           */
      response_list=resp response_list       /*                           */
      m=words(response_list)                 /*                           */
      n=m%2                                  /*                           */
      if n=0 Then n=1                        /*                           */
      mean=word(response_list,n)             /*                           */
      Return mean                            /*                           */
    End                                      /*                           */
  If resp> last_word Then                    /*                           */
    Do                                       /*                           */
      response_list=response_list resp       /*                           */
      m=words(response_list)                 /*                           */
      n=m%2                                  /*                           */
      if n=0 Then n=1                        /*                           */
      mean=word(response_list,n)             /*                           */
      Return mean                            /*                           */
    End                                      /*                           */

  Do R=1 to Count                            /*                           */
     here=word(response_list,R)              /*                           */
     Select                                  /*                           */
     When resp>here Then                     /*                           */
       do                                    /*                           */
         new_list=new_list here              /*                           */
       End                                   /*                           */
     When resp<here Then                     /*                           */
       Do                                    /*                           */
         new_list=new_list resp subword(response_list,R)
         response_list=new_list              /*                           */
         m=words(response_list)              /*                           */
         n=m%2                               /*                           */
         if n=0 Then n=1                     /*                           */
         mean=word(response_list,n)          /*                           */
         Return mean                         /*                           */
       End                                   /*                           */
     When resp=here Then                     /*                           */
       Do                                    /*                           */
         new_list=new_list resp subword(response_list,R)
         response_list=new_list              /*                           */
         m=words(response_list)              /*                           */
         n=m%2                               /*                           */
         if n=0 Then n=1                     /*                           */
         mean=word(response_list,n)          /*                           */
         Return mean                         /*                           */
       End                                   /*                           */
     Otherwise                               /*                           */
     End                                     /*                           */
  End                                        /*                           */
  response_list=new_list                     /*                           */
  m=words(response_list)                     /*                           */
  n=m%2                                      /*                           */
  if n=0 Then n=1                            /*                           */
  mean=word(response_list,n)                 /*                           */
Return mean                                  /*                           */
/*--------------------------------------------------------------------*/
/* CHECKECI     - Check the eci.return_code passed and report         */
/*--------------------------------------------------------------------*/
checkeci: Procedure
Arg eci_rc .
Select                                      /* Check of non-zero rcs */
When eci_rc ='-1'  Then
      say 'rc='eci_rc 'ECI-ERR-INVALID-DATA-LENGTH'
When eci_rc ='-2'  Then
      say 'rc='eci_rc 'ECI-ERR-INVALID-EXTEND-MODE'
When eci_rc ='-3'  Then
      say 'rc='eci_rc 'ECI-ERR-NO-CICS'
When eci_rc ='-4'  Then
      say 'rc='eci_rc 'ECI-ERR-CICS-DIED'
When eci_rc ='-5'  Then
      say 'rc='eci_rc 'ECI-ERR-REQUEST-TIMEOUT'
When eci_rc ='-6'  Then
      say 'rc='eci_rc 'ECI-ERR-RESPONSE-TIMEOUT'
When eci_rc ='-7'  Then
    Do
      say 'rc='eci_rc 'ECI-ERR-TRANSACTION-ABEND'
      say 'ECI Abend Code='eci.abend_code
    End
When eci_rc ='-8'  Then
      say 'rc='eci_rc 'ECI-ERR-EXEC-NOT-RESIDENT or ECI-ERR-LUW-TOKEN'
When eci_rc ='-9'  Then
      say 'rc='eci_rc 'ECI-ERR-SYSTEM-ERROR'
When eci_rc ='-10' Then
      say 'rc='eci_rc 'ECI-ERR-NULL-WIN-HANDLE'
When eci_rc ='-12' Then
      say 'rc='eci_rc 'ECI-ERR-NULL-MESSAGE-ID'
When eci_rc ='-13' Then
      say 'rc='eci_rc 'ECI-ERR-THREAD-CREATE-ERROR'
When eci_rc ='-14' Then
      say 'rc='eci_rc 'ECI-ERR-INVALID-CALL-TYPE'
When eci_rc ='-15' Then
      say 'rc='eci_rc 'ECI-ERR-ALREADY-ACTIVE'
When eci_rc ='-16' Then
      say 'rc='eci_rc 'ECI-ERR-RESOURCE-SHORTAGE'
When eci_rc ='-17' Then
      say 'rc='eci_rc 'ECI-ERR-NO-SESSIONS'
When eci_rc ='-18' Then
      say 'rc='eci_rc 'ECI-ERR-NULL-SEM-HANDLE'
When eci_rc ='-19' Then
      say 'rc='eci_rc 'ECI-ERR-INVALID-DATA-AREA  '
Otherwise
  say 'rc='eci_rc 'is unknown'
End
Return
/*-----------------------------------------------------------------*/
/* RexxSetup - Ensure RexxSetup is loaded                          */
/*-----------------------------------------------------------------*/
RexxSetup: Procedure
  Call RxFuncAdd 'SysLoadFuncs', 'RexxUtil', 'SysLoadFuncs'
  Call SysLoadFuncs
Return 0
/*-----------------------------------------------------------------*/
/* EciSetup - Ensure RxCICS is loaded                              */
/*-----------------------------------------------------------------*/
EciSetup: Procedure
  If rxfuncquery('ECI') Then Do
     If rxfuncadd('ECI', 'RXECI', 'RxECI' ) Then signal failure
     say "ECI registered"
  End
Return 0
/*-----------------------End of HECHO.CMD--------------------------*/
