/* TESTRUN.CMD */
/*******************************************************************/
/*                                                                 */
/* MODULE NAME      TESTRUN.CMD                                    */
/*                                                                 */
/* DESCRIPTIVE NAME Drives LECHO, HECHO, and TECHD CMD files       */
/*                                                                 */
/* Statement:       Licensed Materials - Property of IBM           */
/*                                                                 */
/*                  CA31 SupportPac                                */
/*                  (c) Copyright IBM Corp. 1988, 1995.            */
/*                                                                 */
/*                  See Copyright Instructions.                    */
/*                                                                 */
/*                  All rights reserved.                           */
/*                                                                 */
/*                  U.S. Government Users Restricted Rights - use, */
/*                  duplication or disclosure restricted by GSA    */
/*                  ADP Schedule Contract with IBM Corp.           */
/*                                                                 */
/* Status:          Version 1 Release 1                            */
/*                                                                 */
/* Modified:        11 April 1995                                  */
/* Status:          Added instructions for running with CICS Client*/
/*                  for OS/2.  (Start/Stop with CICSCLI.EXE)       */
/*  NOTES :-                                                       */
/*    DEPENDENCIES = OS/2 V2.x, CICS OS/2, RXECI                   */
/*                   None                                          */
/*    RESTRICTIONS = none                                          */
/*    MODULE TYPE  = Command file                                  */
/*    PROCESSOR    = PS/2 and PC                                   */
/*                                                                 */
/*******************************************************************/
/*-----------------------------------------------------------------*/
/*  TESTRUN - This is the Main Interface for client testing        */
/*                                                                 */
/*-----------------------------------------------------------------*/
/*  Requirements:                                                  */
/*  -------------                                                  */
/*  This function makes use of RexxUtil that comes with OS/2 V2.x  */
/*  It also executes the FAARQPAL batch interface provided by      */
/*  the CICS client.  In addition, the DLLs used by the CICS Client*/
/*  must be available -- either by having those DLLs in a directory*/
/*  defined on the LIBPATH or in the same directory with this      */
/*  sample program.  Also, the CICS client executables will need   */
/*  to be in a directory defined in the SET PATH or in the same    */
/*  directory as this function.                                    */
/*                                                                 */
/*  The command file is set up to run with the Distributed Client  */
/*  feature of CICS for OS/2.  To run with CICS Client for OS/2,   */
/*  the statements used to start and stop the client must be       */
/*  modified to use the correct executable.  See the entries in    */
/*  Start_Control and Stop_Control for what needs to be changed.   */
/*                                                                 */
/*  In addition, this command also will make ECI calls to the      */
/*  ECHO.DLL program on CICS OS/2 for the purpose of monitoring    */
/*  the status of the server.                                      */
/*                                                                 */
/*  The additional commands (shown below) have a common structure. */
/*  If the Client Control panel is terminated, then these          */
/*  commands will also terminate thier OS/2 windows.               */
/*                                                                 */
/*-----------------------------------------------------------------*/
/*  Command Input                                                  */
/*  -------------                                                  */
/*  There are three options for this command:                      */
/*                                                                 */
/*  Clients - A number between 1 and 15.  This is the number       */
/*            of test commands to be started.                      */
/*                                                                 */
/*  Option  - A single letter, as follows.                         */
/*                                                                 */
/*  L = Local Echo  - invokes the LECHO.CMD                        */
/*  H = Host Echo   - invokes the HECHO.CMD                        */
/*  V = Vsam access - invokes the TECHD.CMD - this uses the backend*/
/*                    VSAMSERV.DLL as well as the TECHBASE.BTR     */
/*                                                                 */
/*  Logfile                                                        */
/*  The three cmd files have the option to log response information*/
/*  You may specify a place and name for these log files.          */
/*  The name MUST be 6 characters or less since the client number  */
/*  will be automatically added to the name.  The directory        */
/*  must also exist.  For example D:\TEST\MYLOG.RSP with three     */
/*  clients selected will result in three files (MYLOG01.RSP,      */
/*  MYLOG02.RSP, and MYLOG03.RSP) being created in the D:\TEST     */
/*  directory.                                                     */
/*                                                                 */
/*-----------------------------------------------------------------*/
DEBUG=0                                      /* Used by RxECI      */
   Trace 'N'                                 /* No Tracing         */
   Parse value '1' With first option_cmd     /* Initialize         */
   Arg Clients Option LogFile . /* Allow for Options and Log File  */
   If clients <1 | Clients > 15 Then         /* Begin Verification */
     Do
        Say "I don't understand your input"
        Parse Value '' with Clients Option LogFile
        first=0                 /* Allow for selection by the user */
     End
/*-----------------------------------------------------------------*/
    Call RexxSetup                      /* Ensure RexxUtil loaded  */
    Call EciSetup                       /* Ensure RxCICS loaded    */
    Call Start_Control                  /* Start the Control Intrf */
    Call Available                      /* Wait till Up            */
    Call Start_Others                   /* Start other Cmds        */
    Call Monitor                        /* Monitor the health      */
    Call Stop_Control                   /* Start the Control Intrf */
Exit                                    /* Terminate               */
/*-----------------------------------------------------------------*/
/* RexxSetup - Ensure RexxSetup is loaded                          */
/*-----------------------------------------------------------------*/
RexxSetup: Procedure
  Call RxFuncAdd 'SysLoadFuncs', 'RexxUtil', 'SysLoadFuncs'
  Call SysLoadFuncs
Return 0
/*-----------------------------------------------------------------*/
/* EciSetup - Ensure RxCICS is loaded                              */
/*-----------------------------------------------------------------*/
EciSetup: Procedure
  If rxfuncquery('ECI') Then
    Do
       If rxfuncadd('ECI', 'RXECI', 'RxECI' ) Then Signal failure
       Say "ECI registered"
    End
Return 0
/*-----------------------------------------------------------------*/
/* StartControl - Start up the Client transport                    */
/*-----------------------------------------------------------------*/
Start_Control:
  /*---------------------------------------------------------------*/
  /*     Start the batch interface to start the Client Interface   */
  /*     Options are:                                              */
  /*                   /Q = No prompting                           */
  /*                   /S = Start the Client                       */
  /*---------------------------------------------------------------*/
  /*'@CICSCLI /S /Q'      USE THIS ENTRY FOR CICS CLIENT FOR OS/2  */
    '@FAARQPAL /S /Q'  /* USE THIS ENTRY FOR DISTRIBUTED CLIENT    */
Return rc
/*-----------------------------------------------------------------*/
/* Look for the Server being available                             */
/*-----------------------------------------------------------------*/
Available: Procedure
  Say 'Waiting for the Client Control Interface to activate'
  ECI.program_name      = "ECHO"             /* required           */
  commareaR=Left('MyEchoTest',50)            /* input for ECI call */
  a=time('R')                                /* start the timer    */
  Do T=1 to 15                               /* Try for a while    */
    commarea2 = ECI('ECI.',commareaR)        /* make the call      */
    If ECI.return_code=0 Then                /* Server is Up       */
      Do
         Say Left(commarea2,50)              /* Results of the ECI */
         Say 'CICS OS/2 is now available.'   /* Information        */
         Return                              /* and go back        */
      End
    wait=2+T                                 /* vary the wait time */
    call SysSleep wait                       /* sleep              */
  End
  b=time('E')                                /* stop the timer     */
  Say 'After' b 'seconds, the interface to CICS is still not active'
  Exit                                       /* TERMINATE          */
/*-----------------------------------------------------------------*/
/*  Start up the other commands, clients, etc                      */
/*-----------------------------------------------------------------*/
Start_Others:                           /* Start Other Processes   */
    If first Then                       /* Check for options given */
      Do                                /* and being valid         */
        first=0                         /*                         */
        Parse Value Validate_Option(option) With option_cmd
        If option_cmd <> '' Then Call StartClients
        Else
          Do
             Say 'Selection ('option') not valid, resubmit'
             Exit
          End
      End
    Else
      Do
        /*-----------------------------------------------------------------*/
        /*  Provide a List of Options                                      */
        /*-----------------------------------------------------------------*/
        /*  n = Number of Clients                                          */
        /*  L = Local Echo - invokes the LECHO.CMD                         */
        /*  H = Host Echo  - invokes the HECHO.CMD                         */
        /*  V = Vsam access - invokes the TECHD.CMD - this uses the backend*/
        /*                    program from the TECHDemo.                   */
        /*-----------------------------------------------------------------*/
        Say 'TestRun - Option Selection'
        If clients < 0 | clients > 15 Then
          Do
            Say '---------------------------------------------------'
            Say 'Enter the number of clients to start; (1 to 15)'
            Say 'Use 0 to stop this cmd file'
            Pull clients .
            If clients=0 Then Exit
          End
        If LogFile='' Then
          Do
            Say '-------------------------------------------------------'
            Say 'Enter the name of the logfile to be used by each client'
            Pull LogFile .
          End
        Say '--------------------------------------------------------------'
        Say 'Select Test Option from this list: (enter the character shown)'
        Say 'L = Local Echo   (LECHO.CMD)'
        Say 'H = Host Echo    (HECHO.CMD)'
        Say 'V = Vsam access  (TECHD.CMD)'
        Pull Option .
        Parse Value Validate_Option(option) With option_cmd
        If option_cmd <> '' Then Call StartClients
     End
Return
/*-----------------------------------------------------------------*/
/* Validate Options and Return the CMD file name                   */
/*-----------------------------------------------------------------*/
Validate_Option: Procedure
  Arg Option
  option_cmd=''
  Select
    When Option='L' Then option_cmd='LECHO'
    When Option='H' Then option_cmd='HECHO'
    When Option='V' Then option_cmd='TECHD'
    Otherwise            option_cmd=''
  End
Return option_cmd

/*-----------------------------------------------------------------*/
/* Monitor the health of CICS OS/2 Server via the Local Echo       */
/*-----------------------------------------------------------------*/
Monitor: Procedure
  Parse Value '0 0 0 0 0' With high i bs t sleep
  ECI.program_name      = "ECHO"        /* required                */
  Call SysCls                           /* clear the screen        */
  started=time() date()                 /* When monitor started    */
  say 'Monitor started at' started      /* information message     */
  commareaR=Left('MyEchoTest',50)       /* input for ECI call      */
  sleep=0                               /* first time only         */
  Do Forever
    t=t+1                             /* Count number of ECI calls */
    a=time('R')                       /* start the timer           */
    commarea2 = ECI('ECI.',commareaR) /* make the call             */
    b=time('E')                       /* stop the timer            */
    call SysCurPos '9', '0'           /* reposition the screen     */
    say commarea2                     /* display ECI results       */
    If b>high Then high=b             /* Save the longest time     */
    bs=bs+b                           /* add to the total time     */
    call ecirc                        /* Check the ECI return code */
    sleep='60'                        /* Once a Minute             */
    call SysSleep sleep               /* sleep                     */
    If ECI.return_code<>0 Then Leave  /* If non-zero, get out      */
  end
  Say ' Ended' time() date() 'started' started
Return eci.return_code
/*-----------------------------------------------------------------*/
Stop_Control:
/*-----------------------------------------------------------------*/
/*     STOP the batch interface for the Client Interface           */
/*                   /X = Controlled Stop of the Client            */
/*                   /I = Immediate Stop of the Client             */
/*-----------------------------------------------------------------*/
  /*'@CICSCLI /I /Q'      USE THIS ENTRY FOR CICS CLIENT FOR OS/2  */
    '@FAARQPAL /I /Q'  /* USE THIS ENTRY FOR DISTRIBUTED CLIENT    */
Return rc
/*-----------------------------------------------------------------*/
/*  StartClients                                                   */
/*-----------------------------------------------------------------*/
StartClients:
  If Clients>15 Then Clients=15
  If Clients<0  Then Clients=4
  Parse Value '.LOG' With ext extn drive
  If Substr(logfile,2,1)=':' Then
     Parse Value LogFile  With drive ':' rest
  If Length(drive)=1 Then
     drive=drive||':'
  If lastpos('\',rest)>0 Then
     drive=drive||substr(rest,1,lastpos('\',rest))
  logname=filespec('name',logfile)
  extn   =substr(logname,lastpos('.',logname)+1)
  logname=substr(logname,1,lastpos('.',logname)-1)
  If extn  ='' Then extn=ext
  Else extn='.'||extn
  logname=left(logname'xxxxxx',6)
  Do C = 1 to Clients
     logit=drive||logname||right('00'C,2)||extn
     call SysSleep '2'                /* sleep to let clients start*/
     '@START' option_cmd Logit
     Say 'Client number' C 'started'
  End
Return
/*-----------------------------------------------------------------*/
/* general routine                                                 */
/*-----------------------------------------------------------------*/
FAILURE:
  Say 'Failure to add function at line' sigl
  Return 1

/*-----------------------------------------------------------------*/
/*  Check the ECI return code                                      */
/*-----------------------------------------------------------------*/
ecirc:
    say 'Avg ='LEFT((bs/t),12),        /* report average time      */
        'Last='left(b,12),             /* and last time            */
        'sleep=' sleep,                /* sleep time               */
        'longest=' high
    If eci.return_code ='0'   Then     /* if good, goback          */
      Do
         say 'rc='eci.return_code 'ECI-NO-ERROR'
         Return
      End
    CALL CHECKECI eci.return_code
Return
/*--------------------------------------------------------------------*/
/* CHECKECI     - Check the eci.return_code passed and report         */
/*--------------------------------------------------------------------*/
Arg eci_rc .
Select                                      /* Check of non-zero rcs */
When eci_rc ='-1'  Then
      say 'rc='eci_rc 'ECI-ERR-INVALID-DATA-LENGTH'
When eci_rc ='-2'  Then
      say 'rc='eci_rc 'ECI-ERR-INVALID-EXTEND-MODE'
When eci_rc ='-3'  Then
      say 'rc='eci_rc 'ECI-ERR-NO-CICS'
When eci_rc ='-4'  Then
      say 'rc='eci_rc 'ECI-ERR-CICS-DIED'
When eci_rc ='-5'  Then
      say 'rc='eci_rc 'ECI-ERR-REQUEST-TIMEOUT'
When eci_rc ='-6'  Then
      say 'rc='eci_rc 'ECI-ERR-RESPONSE-TIMEOUT'
When eci_rc ='-7'  Then
    Do
      say 'rc='eci_rc 'ECI-ERR-TRANSACTION-ABEND'
      say 'ECI Abend Code='eci.abend_code
    End
When eci_rc ='-8'  Then
      say 'rc='eci_rc 'ECI-ERR-EXEC-NOT-RESIDENT or ECI-ERR-LUW-TOKEN'
When eci_rc ='-9'  Then
      say 'rc='eci_rc 'ECI-ERR-SYSTEM-ERROR'
When eci_rc ='-10' Then
      say 'rc='eci_rc 'ECI-ERR-NULL-WIN-HANDLE'
When eci_rc ='-12' Then
      say 'rc='eci_rc 'ECI-ERR-NULL-MESSAGE-ID'
When eci_rc ='-13' Then
      say 'rc='eci_rc 'ECI-ERR-THREAD-CREATE-ERROR'
When eci_rc ='-14' Then
      say 'rc='eci_rc 'ECI-ERR-INVALID-CALL-TYPE'
When eci_rc ='-15' Then
      say 'rc='eci_rc 'ECI-ERR-ALREADY-ACTIVE'
When eci_rc ='-16' Then
      say 'rc='eci_rc 'ECI-ERR-RESOURCE-SHORTAGE'
When eci_rc ='-17' Then
      say 'rc='eci_rc 'ECI-ERR-NO-SESSIONS'
When eci_rc ='-18' Then
      say 'rc='eci_rc 'ECI-ERR-NULL-SEM-HANDLE'
When eci_rc ='-19' Then
      say 'rc='eci_rc 'ECI-ERR-INVALID-DATA-AREA  '
Otherwise
  say 'rc='eci_rc 'is unknown'
End
Return
/*-------------------End of TESTRUN.CMD------------------------*/
