/* TSTDRIVE.CMD   */
/**************************************************************************/
/*                                                                        */
/* MODULE NAME      TSTDRIVE.CMD                                          */
/*                                                                        */
/* DESCRIPTIVE NAME Sample driver cmd file to drive ECI calls, where the  */
/*                  data and control information is in different files.   */
/*                                                                        */
/* Statement:       Licensed Materials - Property of IBM                  */
/*                                                                        */
/*                  CA31 SupportPac                                       */
/*                  (c) Copyright IBM Corp. 1988, 1994.                   */
/*                                                                        */
/*                  See Copyright Instructions.                           */
/*                                                                        */
/*                  All rights reserved.                                  */
/*                                                                        */
/*                  U.S. Government Users Restricted Rights - use,        */
/*                  duplication or disclosure restricted by GSA           */
/*                  ADP Schedule Contract with IBM Corp.                  */
/*                                                                        */
/* Status:          Version 1 Release 0                                   */
/* Added:           20 April 94                                           */
/*                                                                        */
/*  NOTES :-                                                              */
/*    DEPENDENCIES = OS/2 2.x, CICS OS/2, RxECI                           */
/*                   None                                                 */
/*    RESTRICTIONS = none                                                 */
/*    MODULE TYPE  = Command file                                         */
/*    PROCESSOR    = PS/2 and PC                                          */
/*    DOCUMENTATION= TSTD_DOC.DOC                                         */
/*                                                                        */
/**************************************************************************/
DEBUG = 0                                    /* used by RxECI for debug   */

  Parse Value '0 0 0 0 0 0 0 0 0' With,      /* initialize valiables      */
    high i bs t log inp pro sleep mean_count /* used later                */
  response_list=''                           /* used by mean function     */
  ECI.=''                                    /* start off numm            */
  Arg first second third                     /* Accept user's requests    */
  If first='?' Then Call Help                /* Provide simple help       */
  Call RexxSetup                             /* Ensure RexxUtil loaded    */
  Call SysCls                                /* clear the screen          */
  Call Process_Options                       /* Determine input choices   */
  Call EciSetup                              /* Ensure RxCICS loaded      */
  Call ProcessPro                            /* process the profile       */
  started=time() date()                      /* remember when started     */
  say 'Test started at' started              /* and notify the user       */
  call logit 'Test started at' started       /* and optionally logit      */
  do Forever                                 /*---------------------------*/
    Call GetNext                             /* Get the input information */
    if program='EOF' Then Leave              /* all input processed       */
    if program<>'' then                      /* if program name given     */
       ECI.program_name=program              /* then set ECI parm         */
    commarea=left(commarea,clength)          /* and fix up commarea       */
    ECI.commarea_length = clength            /* set the length            */
    t=t+1                                    /* Count number of ECI calls */
    a=time('R')                              /* start the timer           */
    commarea2 = ECI('ECI.',commarea)         /* make the call             */
    b=time('E')                              /* stop the timer            */
    if b>high then high=b                    /* Save the longest time     */
    bs=bs+b                                  /* add to the total time     */
    call SysCurPos '9', '1'                  /* reposition the screen     */
    say 'ECI Count='right(t,5)               /* make a report             */
    say left(commarea2,50) '(len=' length(commarea2)')    '
    call ecirc                               /* Check the ECI return code */
    If maxsleep > 0 Then                     /* If wait requested         */
       call SysSleep MAXSLEEP                /* sleep sleep               */
    If eci.return_code<>0 Then Leave         /* Run till non-zero         */
  end                                        /*---------------------------*/
  if eci.abend_code<>'' Then                 /* if application abended    */
     call logit  eci.abend_code              /* then log the info         */
  call logit 'Test stopped at' time() date() /* log the test has stopped  */
  Say 'Test stopped at' time() date()        /* and tell the user         */
  exit                                       /* Terminate testing         */
/*------------------------------------------------------------------------*/
/* Error loading RxECI or RxUtils                                         */
/*------------------------------------------------------------------------*/
FAILURE:
  Say 'Failure to add function at line' sigl /* unable to load function   */
  Return 1
/*------------------------------------------------------------------------*/
/* Get the next input record                                              */
/*------------------------------------------------------------------------*/
Getnext:                                     /*---------------------------*/
  record=readfile(inp_file)                  /* get next real record      */
  Parse value record with field1 commarea    /* divide the record         */
  Parse Value field1 with,                   /* parse the control info    */
    program '/' minw '/' maxw '/' clen .     /* into components           */
  parse value '0 0 0' with min_wait max_wait clength/* initialize values  */
  if datatype(minw)='NUM' Then min_wait=minw /* set value   if numeric    */
  if datatype(maxw)='NUM' Then max_wait=maxw /* set value   if numeric    */
  if min_wait='' then max_wait='0'           /* discard max if no min     */
  if max_wait =min_wait then max_wait='0'    /* discard max if same       */
  if max_wait<=min_wait then max_wait='0'    /* discard max if not >      */
  maxsleep=0                                 /* Assume no wait time       */
  If max_wait='0' & min_wait>0 then          /* check for a fixed         */
     maxsleep=min_wait                       /* wait time                 */
  Else maxsleep=random(min_wait,max_wait)    /* else use random time      */
  If datatype(clen)='NUM' Then clength =clen /* determine length of       */
  else clength=length(commarea)              /* commarea                  */
Return                                       /*---------------------------*/
/*------------------------------------------------------------------------*/
/* Read a Record                                                          */
/*------------------------------------------------------------------------*/
readfile: Procedure                          /* general read routine      */
arg fid .                                    /* name of the file to read  */
Do Until lines(fid)=0                        /* a loop that we'll stop    */
   record=linein(fid)                        /* physical read             */
   If left(record,1)='*' then iterate        /* ignore comment lines      */
   If record        =''  then iterate        /* and blank lines           */
   /*---------------------------------------------------------------------*/
   /* Special Processing for Records can occur here                       */
   /*                                                                     */
   /* For example, to add the program/min_wait/max_wait/commarealength    */
   /* based on the content of the record or to ignore specific records.   */
   /*                                                                     */
   /*---------------------------------------------------------------------*/
   Return record                             /* or return what is found   */
End                                          /* end of loop               */
blah=lineout(fid)                            /* now close the file        */
return 'EOF'                                 /* all out of records        */
/*------------------------------------------------------------------------*/
/* Process the profile, Set some defaults and then handle the profile     */
/*------------------------------------------------------------------------*/
ProcessPro:
  ECI.program_name       = 'ECHO    '        /* program name              */
  ECI.userid             = 'SYSAD   '        /* userid                    */
  ECI.password           = 'SYSAD   '        /* password                  */
  ECI.call_type          = 'ECI_SYNC'        /* call_type                 */
  ECI.timeout            = '60'              /* timeout                   */
  ECI.extend_mode        = 'ECI_NO_EXTEND'   /* extended_mode             */
  If pro Then                                /* if profile requested      */
    do until lines(pro_file)=0               /* loop and read             */
       proline=linein(pro_file)              /* Do actual Read            */
       proline=strip(proline)                /*  remove blanks            */
       If left(proline,1)='*' then iterate   /*  is this a comment        */
       If proline        =''  then iterate   /*  or blank                 */
       Interpret proline                     /* execute the line          */
    end                                      /*---------------------------*/
    blah=lineout(pro_file)                   /* now close the file        */
Return
/*------------------------------------------------------------------------*/
/*  Check the ECI return code and format the log record                   */
/*------------------------------------------------------------------------*/
ecirc:                                       /*---------------------------*/
  avg=bs/t                                   /* compute average resp      */
  avg=format(avg,2,3)                        /* and format it             */
  last=format(b,2,3)                         /* remember the last         */
  b=format(b,2,3)                            /*                           */
  meant=mean(b)                              /* compute the mean for last */
  high=format(high,2,3)                      /* 500 values                */
  say 'Avg ='avg,                            /* report average time       */
      'Last='last,                           /* and last time             */
      'mean='meant,                          /*                           */
      'sleep=' sleep,                        /*                           */
      'longest=' high                        /*                           */
  If Log Then                                /*                           */
    Do                                       /*                           */
      log_rec=log_file Left(eci.program_name,8),
      right(t,8) left(avg,6) left(b,6) left(meant,6) right(sleep,2),
      left(high,6) right(clength,4) right(eci.return_code,3),
      time() date() eci.abend_code
      call logit log_rec                     /* log the info              */
    End                                      /*                           */
  If eci.return_code ='0'   Then             /* if good, goback           */
     say 'rc='eci.return_code 'ECI-NO-ERROR' /*                          */
  Else Call CHECKECI eci.return_code         /* else do more checking     */
Return
/*------------------------------------------------------------------------*/
/* Compute the mean response time for 500 values only to minimize overhead*/
/*------------------------------------------------------------------------*/
Mean: Procedure Expose response_list mean_count
  Arg resp .                                 /*                           */
  new_list=''                                /*                           */
  mean_count=mean_count+1                    /* count the number of values*/
  If mean_count>499 Then                     /* use up to 500 values      */
    Do                                       /* to kee[ down overhead     */
      mean_count=0                           /* reset the count           */
      response_list=''                       /* and the list              */
    End                                      /*                           */
  If response_list='' Then                   /*                           */
    Do                                       /*                           */
      response_list=resp                     /*                           */
      Return resp                            /*                           */
    End                                      /*                           */
  Count=words(response_list)                 /*                           */
  first_word=word(response_list,1)           /*                           */
  last_word =word(response_list,Count)       /*                           */
  If resp<first_word Then                    /*                           */
    Do                                       /*                           */
      response_list=resp response_list       /*                           */
      m=words(response_list)                 /*                           */
      n=m%2                                  /*                           */
      if n=0 Then n=1                        /*                           */
      mean=word(response_list,n)             /*                           */
      Return mean                            /*                           */
    End                                      /*                           */
  If resp> last_word Then                    /*                           */
    Do                                       /*                           */
      response_list=response_list resp       /*                           */
      m=words(response_list)                 /*                           */
      n=m%2                                  /*                           */
      if n=0 Then n=1                        /*                           */
      mean=word(response_list,n)             /*                           */
      Return mean                            /*                           */
    End                                      /*                           */

  Do R=1 to Count                            /*                           */
     here=word(response_list,R)              /*                           */
     Select                                  /*                           */
     When resp>here Then                     /*                           */
       do                                    /*                           */
         new_list=new_list here              /*                           */
       End                                   /*                           */
     When resp<here Then                     /*                           */
       Do                                    /*                           */
         new_list=new_list resp subword(response_list,R)
         response_list=new_list              /*                           */
         m=words(response_list)              /*                           */
         n=m%2                               /*                           */
         if n=0 Then n=1                     /*                           */
         mean=word(response_list,n)          /*                           */
         Return mean                         /*                           */
       End                                   /*                           */
     When resp=here Then                     /*                           */
       Do                                    /*                           */
         new_list=new_list resp subword(response_list,R)
         response_list=new_list              /*                           */
         m=words(response_list)              /*                           */
         n=m%2                               /*                           */
         if n=0 Then n=1                     /*                           */
         mean=word(response_list,n)          /*                           */
         Return mean                         /*                           */
       End                                   /*                           */
     Otherwise                               /*                           */
     End                                     /*                           */
  End                                        /*                           */
  response_list=new_list                     /*                           */
  m=words(response_list)                     /*                           */
  n=m%2                                      /*                           */
  if n=0 Then n=1                            /*                           */
  mean=word(response_list,n)                 /*                           */
Return mean                                  /*                           */
/*------------------------------------------------------------------------*/
/* CHECKECI     - Check the eci.return_code passed and report             */
/*------------------------------------------------------------------------*/
checkeci: Procedure
  Arg eci_rc .
  Select
  When eci_rc ='-1'  Then
        say 'rc='eci_rc 'ECI-ERR-INVALID-DATA-LENGTH'
  When eci_rc ='-2'  Then
        say 'rc='eci_rc 'ECI-ERR-INVALID-EXTEND-MODE'
  When eci_rc ='-3'  Then
        say 'rc='eci_rc 'ECI-ERR-NO-CICS'
  When eci_rc ='-4'  Then
        say 'rc='eci_rc 'ECI-ERR-CICS-DIED'
  When eci_rc ='-5'  Then
        say 'rc='eci_rc 'ECI-ERR-REQUEST-TIMEOUT'
  When eci_rc ='-6'  Then
        say 'rc='eci_rc 'ECI-ERR-RESPONSE-TIMEOUT'
  When eci_rc ='-7'  Then
      Do
        say 'rc='eci_rc 'ECI-ERR-TRANSACTION-ABEND'
        say 'ECI Abend Code='eci.abend_code
      End
  When eci_rc ='-8'  Then
        say 'rc='eci_rc 'ECI-ERR-EXEC-NOT-RESIDENT or ECI-ERR-LUW-TOKEN'
  When eci_rc ='-9'  Then
        say 'rc='eci_rc 'ECI-ERR-SYSTEM-ERROR'
  When eci_rc ='-10' Then
        say 'rc='eci_rc 'ECI-ERR-NULL-WIN-HANDLE'
  When eci_rc ='-12' Then
        say 'rc='eci_rc 'ECI-ERR-NULL-MESSAGE-ID'
  When eci_rc ='-13' Then
        say 'rc='eci_rc 'ECI-ERR-THREAD-CREATE-ERROR'
  When eci_rc ='-14' Then
        say 'rc='eci_rc 'ECI-ERR-INVALID-CALL-TYPE'
  When eci_rc ='-15' Then
        say 'rc='eci_rc 'ECI-ERR-ALREADY-ACTIVE'
  When eci_rc ='-16' Then
        say 'rc='eci_rc 'ECI-ERR-RESOURCE-SHORTAGE'
  When eci_rc ='-17' Then
        say 'rc='eci_rc 'ECI-ERR-NO-SESSIONS'
  When eci_rc ='-18' Then
        say 'rc='eci_rc 'ECI-ERR-NULL-SEM-HANDLE'
  When eci_rc ='-19' Then
        say 'rc='eci_rc 'ECI-ERR-INVALID-DATA-AREA  '
  Otherwise
    say 'rc='eci_rc 'is unknown'
  End
Return
/*------------------------------------------------------------------------*/
/* RexxSetup - Ensure RexxSetup is loaded                                 */
/*------------------------------------------------------------------------*/
RexxSetup: Procedure                           /*                         */
  Call RxFuncAdd 'SysLoadFuncs', 'RexxUtil', 'SysLoadFuncs'
  Call SysLoadFuncs
Return 0
/*------------------------------------------------------------------------*/
/* EciSetup - Ensure RxCICS is loaded                                     */
/*------------------------------------------------------------------------*/
EciSetup: Procedure                            /*                         */
  If rxfuncquery('ECI') Then Do
     If rxfuncadd('ECI', 'RXECI', 'RxECI' ) Then signal failure
     say "ECI registered"                      /*                         */
  End
Return 0
/*------------------------------------------------------------------------*/
/* Logit - writes records to the log file                                 */
/*------------------------------------------------------------------------*/
Logit: Procedure Expose log_file log           /*                         */
  Arg message                                  /* get the message         */
  if log then                                  /* if logging active       */
     '@ECHO' message '>>' log_file             /* then logit              */
Return
/*------------------------------------------------------------------------*/
/* Process user options                                                   */
/*------------------------------------------------------------------------*/
Process_Options:
parse value '' with pro_file log_file inp_file
  Select                                       /*                         */
    When left(first,3) ='/P:' then pro_file=substr(first,4)
    When left(first,3) ='/L:' then log_file=substr(first,4)
    When left(first,3) ='/I:' then inp_file=substr(first,4)
    Otherwise
  End
  Select                                       /*                         */
    When left(second,3)='/P:' then pro_file=substr(second,4)
    When left(second,3)='/L:' then log_file=substr(second,4)
    When left(second,3)='/I:' then inp_file=substr(second,4)
    Otherwise
  End
  Select                                       /*                         */
    When left(third,3) ='/P:' then pro_file=substr(third,4)
    When left(third,3) ='/L:' then log_file=substr(third,4)
    When left(third,3) ='/I:' then inp_file=substr(third,4)
    Otherwise
  End
  If log_file<>'' Then                         /*                         */
    do                                         /*                         */
      log=1                                    /*                         */
      Say 'logging to:' log_file               /*                         */
    End                                        /*                         */
  Else log=0                                   /*  No logging to be done  */
  If pro_file<>'' Then                         /*                         */
    do                                         /*                         */
      pro=1                                    /*                         */
      Say 'Profile is:' pro_file               /*                         */
    End                                        /*                         */
  If inp_file<>'' Then                         /*                         */
    do                                         /*                         */
      inp=1                                    /*                         */
      Say 'Input from:' inp_file               /*                         */
    End                                        /*                         */
  Else                                         /*                         */
    do                                         /*                         */
      Say 'An input file was not specified, please resubmit'
      say 'your options were' first second third
      say ''
      call help                                /*                         */
    end                                        /*                         */
Return
/*------------------------------------------------------------------------*/
/* Provide Help                                                           */
/*------------------------------------------------------------------------*/
Help: Procedure

  Say 'TSTDRIVE.CMD - A Sample to show how to automate the testing'
  Say '               of the Server part of a Client-Server application.'
  Say ''
  Say 'input options are as follows:'
  say ''
  Say '   /P:file_name - this is the name of the profile'
  Say '   /I:file_name - this is the name of the ECI-input file'
  Say '   /L:file_name - this is the name of the log file'
  say ''
  Say 'See the TSTDRIVE.DOC for additional details.'

Exit
/*--------------------End of TSTDRIVE.CMD---------------------------------*/
